import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.nttdocomo.fs.File;
import com.nttdocomo.fs.Folder;
import com.nttdocomo.io.FileEntity;
import com.nttdocomo.ui.MediaImage;
import com.nttdocomo.ui.MediaManager;


public class User {
	public static int USER_LIMIT_MAX = 50;
	public static int USER_LIMIT_MIN = 30;
	protected static Hashtable users = new Hashtable();

	protected String id;
	protected String screenName;
	protected MediaImage image;
	protected Date lastAccess;
	
	public String getId() {
		return id;
	}
	public void setId(String id) {
		this.id = id;
	}
	public MediaImage getImage() {
		return image;
	}
	public void setImage(MediaImage image) {
		this.image = image;
	}
	public String getScreenName() {
		return screenName;
	}
	public void setScreenName(String screenName) {
		this.screenName = screenName;
	}
	public static User getUser(String id) {
		return (User) users.get(id);
	}
	public Date getLastAccess() {
		return lastAccess;
	}
	public void setLastAccess(Date lastAccess) {
		this.lastAccess = lastAccess;
	}
	
	public static User getUser(String id, String screenName, String imageUrl) {
		User user = (User) users.get(id);
		if (user == null) {
			user = createUser(id, screenName, imageUrl);
		}
		user.setLastAccess(new Date());
		return user;
	}
	public static void flush() {
		if (users.size() <= USER_LIMIT_MAX) {
			return;
		}
		Vector v = new Vector(users.size());
		Enumeration e = users.elements();
		while (e.hasMoreElements()) {
			User u = (User) e.nextElement();
			int i;
			for (i = 0; i < v.size(); i++) {
				User t = (User) v.elementAt(i);
				if (u.getLastAccess().getTime() < t.getLastAccess().getTime()) {
					break;
				}
			}
			v.insertElementAt(u, i);
		}
		for (int i = 0; i < v.size() - USER_LIMIT_MIN; i++) {
			User u = (User) v.elementAt(i);
			removeUser(u.getId());
		}
	}
	protected static User createUser(String id, String screenName, String imageUrl) {
		User user = new User();
		user.setId(id);
		user.setScreenName(screenName);
		MediaImage image = loadImage(id);
		if (image == null) {
			image = getImage(id, imageUrl);
		}
		user.setImage(image);
		users.put(id, user);
		return user;
	}
	protected static void removeUser(String id) {
		User user = (User) users.get(id);
		if (user != null) {
			users.remove(id);
			MediaImage mi = user.getImage();
			if (mi != null) {
				mi.dispose();
			}
		}
	}
	protected static MediaImage loadImage(String id) {
		try {
			Folder folder = FileManager.getFolder();
			File file = folder.getFile(id);
			FileEntity fe = file.open(File.MODE_READ_ONLY);
			InputStream in = fe.openInputStream();
			MediaImage image = MediaManager.getImage(in);
			image.use();
			in.close();
			fe.close();
			return image;
		} catch (IOException e) {
			return null;
		}
	}
	protected static void saveImage(String id, byte data[]) {
		try {
			Folder folder = FileManager.getFolder();
			try {
				File file = folder.getFile(id);
				file.delete();
			} catch (IOException e) {
				// 
			}
			File file = folder.createFile(id);
			FileEntity fe = file.open(File.MODE_WRITE_ONLY);
			OutputStream out = fe.openOutputStream();
			out.write(data);
			out.close();
			fe.close();
		} catch (IOException e) {
			// 
		}
	}
	protected static MediaImage getImage(String id, String url) {
		MediaImage mi = null;
		byte data[] = CommControl.getInstance().getImage(url);
		if (data != null) {
			try {
				mi = MediaManager.getImage(data);
				mi.use();
				saveImage(id, data);
			} catch (Exception e) {
				// 
				mi = null;
			}
		}

		return mi;
	}
}
